--[[ Linkerize: adds control codes to links pasted into supported EditBoxes ]]
-- 08/29/17 011 toc update for 7.3
-- 03/28/17 010 toc update for 7.2
-- 10/24/16 009 toc update for 7.1
-- 07/16/16 008 toc update for 7.0
-- 06/22/15 007 toc update for 6.2
-- 02/24/15 006 toc update for 6.1
-- 10/14/14 005 toc update for 6.0
-- 09/11/13 004 toc update for 5.4
-- 05/21/13 003 toc update for 5.3
-- 08/27/12 002 bug fix
-- 02/03/12 001 initial release

local version = 006 -- increment this if you change this file

local supported_editboxes = {
	"ChatFrame1EditBox", -- chat entry box
	"TinyPadEditBox", -- TinyPad main window
	"MacroFrameText", -- macro edit window
	-- add other ones here
}

if not Linkerize or Linkerize.version_loaded<version then

	local linkerize=CreateFrame("Frame","Linkerize")

	linkerize.version_loaded = version
	linkerize.known_links = {}
	linkerize.last_change = 0

	linkerize:SetScript("OnEvent",function(self,event,addon)
		if event=="PLAYER_LOGIN" then
			for _,editBox in pairs(supported_editboxes) do
				if _G[editBox] then
					linkerize:Register(_G[editBox])
				end
			end
		elseif event=="ADDON_LOADED" and addon=="Blizzard_MacroUI" then
			-- special case for MacroFrameText, doesn't exist until it loads
			linkerize:Register(MacroFrameText)
		end
	end)
	linkerize:RegisterEvent("PLAYER_LOGIN")
	linkerize:RegisterEvent("ADDON_LOADED")

	function linkerize.NoteAnyLinks(self) -- store any clickable links into known_links
		local body = self:GetText()
		local link_pattern="\124c%x%x%x%x%x%x%x%x\124H.-\124h%[.-%]\124h\124r"
		local name_pattern = "\124c%x%x%x%x%x%x%x%x\124H.-\124h(%[.-%])\124h\124r"
		for link in body:gmatch(link_pattern) do
			linkerize.known_links[link:match(name_pattern)] = link
		end
	end

	function linkerize.Linkerizer(self) -- convert all unclickable links to known_links

		linkerize.NoteAnyLinks(self)
		if GetTime()-linkerize.last_change < 0.1 then return end

		local body = self:GetText()
		local unclickable_pattern = "%[%C-%]" -- "[^\124][^h](%[%C+%])[^\124][^h]"
		local unclickable_name, stub,done,change_made, first,last
		local thumb = 1
		local cursor_pos = self:GetCursorPosition()
		local original_length = body:len()
		while not done do
			first,last = body:find(unclickable_pattern,thumb)
			if first then
				thumb = last+1
				if body:sub(first-2,first-1)~="\124h" and body:sub(last+1,last+2)~="\124h" then
					unclickable_name = body:sub(first,last)
					if linkerize.known_links[unclickable_name] then
						stub = body:sub(1,first-1)..linkerize.known_links[unclickable_name]
						thumb = stub:len()+1
						stub = stub..body:sub(last+1,-1)
						body = stub
						change_made = 1
					end
				end
			else
				done = 1
			end
		end
		if change_made then
			local new_length = body:len()
			if new_length <= self:GetMaxLetters() then
				self:SetText(body)
				self:SetCursorPosition(cursor_pos + new_length-original_length)
			else
				print("|cFFFF3333Not enough room to paste a clickable link.")
			end
			linkerize.last_change = GetTime()
		end
	end

	function linkerize:Register(editBox)
		if not editBox.linkerized then
			editBox:HookScript("OnEditFocusGained",linkerize.NoteAnyLinks)
			editBox:HookScript("OnTextChanged",linkerize.Linkerizer)
			editBox.linkerized = 1
		end
	end

--[[ Linkerize all the things!
	-- this works but sadly taints all editboxes, meaning macros can't run grr

	local function hookSetText(oldMethod,...)
		return function(self,...)
			if not self.linkerized then
				linkerize:Register(self)
			end
			oldMethod(self,...)
		end
	end

	local function hookIdxEditBox(t,k)
		local baseFunc = metaIdxEditBox[k]
		if k=="SetText" then
			local newFunc = linkerize.metaHook
			if not newFunc then
				newFunc = hookSetText(baseFunc)
				linkerize.metaHook = newfunc
			end
			return newFunc
		end
		return baseFunc
	end

	local meta = getmetatable(ChatFrame1EditBox)
	metaIdxEditBox = meta.__index
	meta.__index = hookIdxEditBox
]]
--[[
local HookCreators = {
	SetText = function(oldMethod,...)
		return function(self,...)
--			print(self,self and self.GetName and self:GetName(),"in the hook SetText")
			if not self.linkerized then
				self.linkerized = 1
				self:HookScript("OnEditFocusGained",linkerize.NoteAnyLinks)
				self:HookScript("OnTextChanged",linkerize.Linkerizer)
			end
			oldMethod(self,...)
		end
	end,
}

HookCache = { }
local metaIdx
local function hookIdx(t, k)
	local baseFunc
	-- metatable method for 2.0 by JoshBorke
	local metaType = type(metaIdx)
	if (metaType == "function") then
		baseFunc = metaIdx(t,k)
		print("function",k)
	elseif (metaType == "table") then
		baseFunc = metaIdx[k]
--		print(time(),"table",k)
		if k=="SetText" then
--			print(time(),metaIdx)
		end
	end
	local needHook = HookCreators[k]
	if (needHook) then
		local newFunc = HookCache[baseFunc]
		if (not newFunc) then
			newFunc = needHook(baseFunc)
			HookCache[baseFunc] = newFunc
		end
		return newFunc
	end
	return baseFunc
end

local metaIdxEditBox
local function hookIdxEditBox(t,k)
	metaIdx = metaIdxEditBox
	return hookIdx(t, k)
end

local meta = getmetatable(ChatFrame1EditBox)
metaIdxEditBox = meta.__index
meta.__index = hookIdxEditBox
]]

end

